<?php

namespace Laravelista\Algo;

use Illuminate\Support\Arr;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Blade;
use Illuminate\Support\Facades\File;
use Illuminate\Support\ServiceProvider as LaravelServiceProvider;

class ServiceProvider extends LaravelServiceProvider
{
    public function boot()
    {
        $this->bootRoutes();

        $this->bootMigrations();

        $this->bootViews();

        $this->bootBladeComponents();

        $this->bootTranslations();

        $this->publishes([
            __DIR__ . '/../config/algo.php' => App::make('path.config') . '/' . 'algo.php',
        ], 'config');
    }

    public function register()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../config/algo.php',
            'algo'
        );

        /**
         * If the translatable config is not published,
         * use values from this package config file.
         *
         * Otherwise, use the values from the translatable
         * config file.
         */
        if (!File::exists(config_path('translatable.php'))) {
            config([
                'translatable.locales' => config('algo.translatable.locales'),
                'translatable.use_fallback' => config('algo.translatable.use_fallback', true),
                'translatable.fallback_locale' => config('algo.translatable.fallback_locale', 'en')
            ]);
        }

        /**
         * Merge picasso dimensions
         *
         * If the picasso config is published, merge this package
         * picasso dimensions with picasso package config dimensions.
         *
         * Else replace picasso config values with this package config values.
         */
        if (File::exists(config_path('picasso.php'))) {
            config([
                'picasso.dimensions' => array_merge_recursive(
                    config('algo.picasso.dimensions'), config('picasso.dimensions')
                )
            ]);
        } else {
            config([
                'picasso.dimensions' => config('algo.picasso.dimensions')
            ]);
        }
    }

    protected function bootRoutes()
    {
        $this->loadRoutesFrom(__DIR__.'/routes.php');
    }

    protected function bootMigrations()
    {
        $this->loadMigrationsFrom(__DIR__ . '/../migrations');

        $this->publishes([
            __DIR__.'/../migrations/' => App::databasePath('migrations')
        ], 'migrations');
    }

    protected function bootViews()
    {
        $this->loadViewsFrom(__DIR__ . '/../resources/views', 'algo');

        $this->publishes([
            __DIR__ . '/../resources/views' => App::resourcePath('views/vendor/algo'),
        ], 'views');
    }

    protected function bootBladeComponents()
    {
        Blade::include('algo::ui.bootstrap4.forms.input', 'input');
        Blade::include('algo::ui.bootstrap4.forms.checkbox', 'checkbox');
        Blade::include('algo::ui.bootstrap4.forms.textarea', 'textarea');
        Blade::include('algo::ui.bootstrap4.forms.select', 'select');
        Blade::include('algo::ui.bootstrap4.forms.submit', 'submit');
        Blade::aliasComponent('algo::ui.bootstrap4.forms.form', 'form');

        Blade::include('algo::ui.bootstrap4.edit.language_switcher', 'languageSwitcher');

        Blade::include('algo::ui.bootstrap4.show.field', 'field');
        Blade::include('algo::ui.bootstrap4.show.tablist', 'tablist');
        Blade::aliasComponent('algo::ui.bootstrap4.show.tabpanel', 'tabpanel');

        Blade::include('algo::ui.bootstrap4.status_alert', 'statusAlert');

        Blade::aliasComponent('algo::ui.bootstrap4.dropdowns.split_button', 'dropdownSplitButton');
    }

    protected function bootTranslations()
    {
        $this->loadTranslationsFrom(__DIR__ . '/../resources/lang', 'algo');

        $this->publishes([
            __DIR__ . '/../resources/lang' => resource_path('lang/vendor/algo'),
        ]);
    }
}